/***************************************************************************
                          mymoneyfile.cpp
                             -------------------
    copyright            : (C) 2000 by Michael Edwardes
    email                : mte@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "mymoneyfile.h"
#ifndef HAVE_CONFIG_H
#define VERSION "UNKNOWN"
#else
#include "config.h"
#endif

#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <qfile.h>
#include <qdatastream.h>
#include <kdebug.h>

#include "mymoneyaccount.h"

#include "mymoney_config.h"

MyMoneyFile::MyMoneyFile(void)
{
  m_createdDate = QDate::currentDate();
  m_lastAccess = QDate::currentDate();
  m_lastModify = QDate::currentDate();
  m_dirty = false;
  m_initialised = m_containsBanks = m_containsAccounts = m_containsTransactions = false;
}

MyMoneyFile::MyMoneyFile(const QString& usern, const QString& userStreet,
 const QString& userTown, const QString& userCounty, const QString& userPostcode, const QString& userTelephone,
 const QString& userEmail, const QDate& createDate)
{
  m_userName = usern;
  m_userStreet = userStreet;
  m_userTown = userTown;
  m_userCounty = userCounty;
  m_userPostcode = userPostcode;
  m_userTelephone = userTelephone;
  m_userEmail = userEmail;
  m_createdDate = createDate;
  m_lastAccess = createDate;
  m_lastModify = createDate;
  m_dirty = false;
  m_passwordProtected=false;
  m_encrypted=false;
  m_initialised = m_containsBanks = m_containsAccounts = m_containsTransactions = false;
}

MyMoneyFile::~MyMoneyFile()
{
}

QString MyMoneyFile::userAddress(void)
{
  // Return an address containing concatenated strings
  QString m_userAddress;
  m_userAddress = m_userStreet + "\n";
  m_userAddress += m_userTown + "\n";
  m_userAddress += m_userCounty + "\n";
  m_userAddress += m_userPostcode + "\n";
  m_userAddress += m_userTelephone + "\n";
  m_userAddress += m_userEmail + "\n";
  return m_userAddress;
}

int MyMoneyFile::saveAllData(const QString& fileName)
{
  QFile f( fileName );
  f.open( IO_WriteOnly );
  QDataStream s( &f );

  // Write a header with a "magic number" and a version
  QString ver(VERSION);
  s << ver;
  s << (Q_INT32)MAGIC;
  if (m_passwordProtected)
    s << (Q_INT32)1;
  else
    s << (Q_INT32)0;
  if (m_encrypted)
    s << (Q_INT32)1;
  else
    s << (Q_INT32)0;
  s << m_password;

  // Is updated if the user chooses to save the file
  m_lastModify = QDate::currentDate();

  // Simple Data
  s << m_userName
    << m_userStreet
    << m_userTown
    << m_userCounty
    << m_userPostcode
    << m_userTelephone
    << m_userEmail
    << m_createdDate
    << m_lastAccess
    << m_lastModify;

  // Read through the three lists writing the count and then the objects
  // in turn
  s << m_bankNames.count();
  for (QStringList::Iterator it = m_bankNames.begin(); it != m_bankNames.end(); ++it)
    s << (*it);

  s << m_categoryList.count();
  MyMoneyCategory *data;
  for (data=m_categoryList.first(); data != 0; data = m_categoryList.next()) {
    s << *data;
  }

  s << m_payeeList.count();
  MyMoneyPayee *payee;
  for ( payee=m_payeeList.first(); payee!=0; payee=m_payeeList.next()) {
    s << *payee;
  }

  s << (Q_INT32)bankCount();
  MyMoneyBank *bank;
  for (bank=bankFirst(); bank!=0; bank=bankNext()) {
    s << *bank;

    s << (Q_INT32)bank->accountCount();
    MyMoneyAccount *account;
    for (account=bank->accountFirst(); account!=0; account=bank->accountNext()) {
      s << *account;

      s << (Q_INT32)account->transactionCount();
      MyMoneyTransaction *transaction;
      for (transaction=account->transactionFirst(); transaction!=0; transaction=account->transactionNext()) {
        s << *transaction;
      }
    }
  }

  f.close();
  m_dirty = false;
  return 0;
}

int MyMoneyFile::readAllData(const QString& fileName)
{
  // Returns an error code in the form of an int.
  // TODO: add descriptive error returns.
  QFile f( fileName );
  f.open( IO_ReadOnly );
  if (!f.isOpen())
    return 3;

  QDataStream s( &f );

  QString version;
  s >> version;
/* In the future I'll have a check for compatible versions, for now rely on the MAGIC...
  if (version != VERSION) {
    // error code
    return 1;
  }
*/
  Q_INT32 magic;
  s >> magic;
  bool foundOldVersion=false;
  int foundVersionID=0;

  if (magic==VERSION_0_3_3) { // Version(s) 0.3.3 and below
    kdWarning() << "Found a money file belonging to versions 0.3.3 and below.";
    foundOldVersion = true;
    foundVersionID = VERSION_0_3_3;
  }
  else if (magic != MAGIC) {
    return 2;
  }

  Q_INT32 tmp;
  s >> tmp;
  if (tmp==1)
    m_passwordProtected = true;
  else
    m_passwordProtected = false;
  s >> tmp;
  if (tmp==1)
    m_encrypted = true;
  else
    m_encrypted = false;

  // if (m_encrypted ==true) {
  //   All data that follows needs to be decrypted
  // }
  s >> m_password;

  // Simple Data
  if (foundOldVersion) { // We need to change how we read in the data
    if (foundVersionID==VERSION_0_3_3) {
      kdWarning() << "Converting from old 0.3.3 release...\n\tRemoving old file::name field";
      QString temp_delete;
      s >> temp_delete;
    }
  }
  s >> m_userName
    >> m_userStreet
    >> m_userTown
    >> m_userCounty
    >> m_userPostcode
    >> m_userTelephone
    >> m_userEmail
    >> m_createdDate
    >> m_lastAccess
    >> m_lastModify;

  m_lastAccess = QDate::currentDate();

  Q_UINT32 bankNames_count=0, categoryList_count=0, payTo_count=0, bank_count=0, account_count=0, transaction_count=0;

  QString buffer;
  QString buffer2;

  // Reading is quite simple as each object 'knows' how to read into itself
  s >> bankNames_count;
  for (unsigned int i=0; i<bankNames_count; i++) {
    s >> buffer;
    addBankName(buffer);
  }

  MyMoneyCategory category;
  s >> categoryList_count;
  for (unsigned int i=0; i<categoryList_count; i++) {
    s >> category;
    addCategory(category.isIncome(), category.name(), category.minorCategories());
  }

  MyMoneyPayee *payee=0;
  s >> payTo_count;
  for (unsigned int i=0; i<payTo_count; i++) {
    if (payee==0) {
      payee = new MyMoneyPayee;
    }
    s >> (*payee);
    addPayee(payee->name(), payee->address(), payee->postcode(), payee->telephone(), payee->email());
    delete payee;
    payee=0;
  }

  MyMoneyBank bankl;
  MyMoneyBank *bankWrite;
  MyMoneyAccount account;
  MyMoneyAccount *accountWrite;
  MyMoneyTransaction transaction;

  s >> bank_count;
  for ( unsigned int j=0; j<bank_count; j++ ) {
    m_containsBanks = true;
    bankl.readAllData(foundVersionID, s);
//    s >> bankl;
    addBank(bankl.name(), bankl.sortCode(), bankl.city(), bankl.street(), bankl.postcode(), bankl.telephone(), bankl.manager());

    bankWrite = bank(bankl);
    if (!bankWrite) {
      kdWarning() << "Unable to get bankWrite";
      return -9;
    }

    s >> account_count;
    for ( unsigned int k=0; k<account_count; k++ ) {
      m_containsAccounts=true;
      account.readAllData(foundVersionID, s);
//      s >> account;
      bankWrite->newAccount(account.name(), account.accountNumber(), account.accountType(), account.description(), account.openingDate(), account.openingBalance(), account.lastReconcile());

      accountWrite = bankWrite->account(account);
      if (!accountWrite) {
        kdWarning() << "Unabel to get accountWrite";
        return -9;
      }

      s >> transaction_count;
      for ( unsigned int l=0; l<transaction_count; l++) {
        m_containsTransactions=true;
        transaction.readAllData(foundVersionID, s);
//        s >> transaction;
        accountWrite->addTransaction(transaction.method(), transaction.number(), transaction.memo(),
            transaction.amount(), transaction.date(), transaction.categoryMajor(), transaction.categoryMinor(),
            transaction.atmBankName(), transaction.payee(), transaction.accountFrom(), transaction.accountTo(),
            transaction.state());
      }
    }
  }

  m_dirty = false;
  f.close();
  return 0;
}

void MyMoneyFile::resetAllData(void)
{
  // Just empty out all data
  QListIterator<MyMoneyBank> it(m_banks);
  for ( ; it.current(); ++it ) {
    MyMoneyBank *bank = it.current();
    bank->clear();
  }
  m_banks.clear();

  m_userName = QString::null;
  m_userStreet = QString::null;
  m_userTown = QString::null;
  m_userCounty = QString::null;
  m_userPostcode = QString::null;
  m_userTelephone = QString::null;
  m_userEmail = QString::null;
  m_bankNames.clear();
  m_createdDate = QDate::currentDate();
  m_lastAccess = QDate::currentDate();
  m_lastModify = QDate::currentDate();

  QListIterator<MyMoneyCategory> it2(m_categoryList);
  for ( ; it2.current(); ++it2 ) {
    MyMoneyCategory *cat = it2.current();
    cat->clear();
  }
  m_categoryList.clear();

	m_payeeList.clear();
  m_initialised = m_containsBanks = m_containsAccounts = m_containsTransactions = false;
	m_dirty = false;
}

MyMoneyBank* MyMoneyFile::bank(const MyMoneyBank& bank)
{
  unsigned int pos;

  if (findBankPosition(bank, pos)) {
    return m_banks.at(pos);
  }
  return 0;
}

MyMoneyBank* MyMoneyFile::bankFirst(void)
{
  return m_banks.first();
}

MyMoneyBank* MyMoneyFile::bankNext(void)
{
  return m_banks.next();
}

MyMoneyBank* MyMoneyFile::bankLast(void)
{
  return m_banks.last();
}

unsigned int MyMoneyFile::bankCount(void)
{
  return m_banks.count();
}

bool MyMoneyFile::removeBank(const MyMoneyBank& bank)
{
  if (m_banks.count()<=1)
    m_containsBanks=false;

  unsigned int pos;
  if (findBankPosition(bank, pos)) {
		m_dirty=true;
    return m_banks.remove(pos);
	}
	
  return false;
}

void MyMoneyFile::addBankName(const QString& val)
{
	m_dirty=true;
  m_bankNames.append(val);
}

void MyMoneyFile::addMajorCategory(const bool income, const QString& val)
{
  if (val.isEmpty() || val == QString::null)
    return;

  QStringList list;
  addCategory(income, val, list);
}

/**
  * This is the real workhorse of the add cateogry stuff.  Everthing else
  * just calls it.
*/
void MyMoneyFile::addMinorCategory(const bool income, const QString& major, const QString& minor)
{
  if (major.isEmpty())
    return;

  QStringList list;
  list.append(minor);
  addCategory(income, major, list);
}

void MyMoneyFile::addCategory(const bool income, const QString& major, QStringList& minors)
{
  MyMoneyCategory *data;
  for (data=m_categoryList.first(); data!=0; data=m_categoryList.next()) {
    if (data->name() == major) {
      data->addMinorCategory(minors);
      if (data->isIncome() != income)
        data->setIncome(income);
      m_dirty=true;
      return;
    }
  }

  MyMoneyCategory *newData = new MyMoneyCategory(income, major, minors);
  int i = 0;

  // Go through the income first
  if (income == true) {
    for (i=0, data=m_categoryList.first(); data!=0; data=m_categoryList.next(),i++) {
      if (data->name() >= major && data->isIncome()==income) {
        m_categoryList.insert(i, newData);
        m_dirty = true;
        return;
      }
    }
    m_categoryList.insert(i, newData);
    m_dirty=true;
  }

  // then expense
  else if (income == false) {
    for (i=0, data=m_categoryList.first(); data!=0; data=m_categoryList.next(),i++) {
      if (data->name() >= major && data->isIncome()==income) {
        m_categoryList.insert(i, newData);
        m_dirty = true;
        return;
      }
    }
    m_categoryList.insert(i, newData);
    m_dirty=true;
  }
}

void MyMoneyFile::addCategory(const bool income, const QString& major, const QString& minor)
{
  if (major.isEmpty() || minor.isEmpty())
    return;

  QStringList list;
  list.append(minor);
  addCategory(income, major, list);
}

QListIterator<MyMoneyCategory> MyMoneyFile::categoryIterator(void)
{
  QListIterator<MyMoneyCategory> it(m_categoryList);
  return it;
}

void MyMoneyFile::removeMajorCategory(const QString& major)
{
  MyMoneyCategory *data;
  for (data=m_categoryList.first(); data!=0; data=m_categoryList.next()) {
    if (data->name() == major) {
      data->removeAllMinors();
      m_categoryList.remove(m_categoryList.at());
      m_dirty=true;
    }
  }
}

void MyMoneyFile::removeMinorCategory(const QString& major, const QString& minor)
{
  MyMoneyCategory *data;
  for (data=m_categoryList.first(); data!=0; data=m_categoryList.next()) {
    if (data->name() == major) {
      data->removeMinorCategory(minor);
      m_dirty=true;
    }
  }
}

void MyMoneyFile::renameMajor(const QString& oldName, const QString& newName)
{
  if (newName.isEmpty())
    return;

  MyMoneyCategory *data;
  for (data=m_categoryList.first(); data!=0; data=m_categoryList.next()) {
    if (data->name() == oldName) {
      data->setName(newName);
      m_dirty = true;
    }
  }
}

void MyMoneyFile::renameMinor(const QString& major, const QString& oldName, const QString& newName)
{
  if (newName.isEmpty())
    return;

  MyMoneyCategory *data;
  for (data=m_categoryList.first(); data!=0; data=m_categoryList.next()) {
    if (data->name() == major) {
      if (data->renameMinorCategory(oldName, newName))
        m_dirty=true;
    }
  }
}

void MyMoneyFile::addPayee(const QString& newPayee, QString address, QString postcode, QString telephone, QString email)
{
  if (newPayee.isEmpty() || newPayee==QString::null)
    return;

  bool found=false;
  MyMoneyPayee *payee;
  for ( payee=m_payeeList.first(); payee!=0; payee=m_payeeList.next()) {
    if (payee->name() == newPayee)
      found = true;
  }

  if (!found) {
    MyMoneyPayee *np = new MyMoneyPayee(newPayee, address, postcode, telephone, email);
    m_payeeList.append(np);
    m_dirty=true;
  }
}

QListIterator<MyMoneyPayee> MyMoneyFile::payeeIterator(void)
{
  QListIterator<MyMoneyPayee> it(m_payeeList);
  return it;
}

MyMoneyBank *MyMoneyFile::addBank(const QString& name, const QString& sortCode, const QString& city,
  const QString& street, const QString& postcode, const QString& telephone, const QString& manager)
{
  MyMoneyBank *bank = new MyMoneyBank(this, name, sortCode, city, street, postcode, telephone, manager);
  m_banks.append(bank);
	m_dirty=true;
  if (m_containsBanks==false)
    m_containsBanks=true;
  return bank;
}

MyMoneyFile::MyMoneyFile(const MyMoneyFile& right)
{
  m_userName = right.m_userName;
  m_userStreet = right.m_userStreet;
  m_userTown = right.m_userTown;
  m_userCounty = right.m_userCounty;
  m_userPostcode = right.m_userPostcode;
  m_userTelephone = right.m_userTelephone;
  m_userEmail = right.m_userEmail;
  m_createdDate = right.m_createdDate;
  m_lastAccess = right.m_lastAccess;
  m_lastModify = right.m_lastModify;
  m_dirty = right.m_dirty;
  m_passwordProtected = right.m_passwordProtected;
  m_encrypted = right.m_encrypted;
  m_password = right.m_password;
  m_banks.clear();
  m_banks = right.m_banks;
  m_bankNames.clear();
  m_bankNames = right.m_bankNames;
  m_payeeList.clear();
  m_payeeList = right.m_payeeList;
  m_categoryList.clear();
  m_categoryList = right.m_categoryList;
  m_initialised = right.m_initialised;
  m_containsBanks = right.m_containsBanks;
  m_containsAccounts = right.m_containsAccounts;
  m_containsTransactions = right.m_containsTransactions;
}

MyMoneyFile& MyMoneyFile::operator = (const MyMoneyFile& right)
{
  m_userName = right.m_userName;
  m_userStreet = right.m_userStreet;
  m_userTown = right.m_userTown;
  m_userCounty = right.m_userCounty;
  m_userPostcode = right.m_userPostcode;
  m_userTelephone = right.m_userTelephone;
  m_userEmail = right.m_userEmail;
  m_createdDate = right.m_createdDate;
  m_lastAccess = right.m_lastAccess;
  m_lastModify = right.m_lastModify;
  m_dirty = right.m_dirty;
  m_passwordProtected = right.m_passwordProtected;
  m_encrypted = right.m_encrypted;
  m_password = right.m_password;
  m_banks.clear();
  m_banks = right.m_banks;
  m_bankNames.clear();
  m_bankNames = right.m_bankNames;
  m_payeeList.clear();
  m_payeeList = right.m_payeeList;
  m_categoryList.clear();
  m_categoryList = right.m_categoryList;
  m_initialised = right.m_initialised;
  m_containsBanks = right.m_containsBanks;
  m_containsAccounts = right.m_containsAccounts;
  m_containsTransactions = right.m_containsTransactions;
  return *this;
}

bool MyMoneyFile::findBankPosition(const MyMoneyBank& bank, unsigned int& pos)
{
  int k=0;

  QListIterator<MyMoneyBank> it(m_banks);
  for ( k=0; it.current(); ++it, k++) {
    if (*it.current()==bank) {
      pos=k;
      return true;
    }
  }
  pos=k;
  return false;
}

void MyMoneyFile::init(void)
{
  m_initialised=true;
	m_dirty=false;
}

bool MyMoneyFile::isInitialised(void)
{
  return m_initialised;
}

bool MyMoneyFile::containsBanks(void)
{
  kdWarning() << "STUB: in MyMoneyFile::containsBanks";
  return m_containsBanks;
}

bool MyMoneyFile::containsAccounts(void)
{
  kdWarning() <<"STUB: in MyMoneyFile::containsAccounts";
  return m_containsAccounts;
}

bool MyMoneyFile::containsTransactions(void)
{
  kdWarning() << "STUB: in MyMoneyFile::containsTransactions";
  return m_containsTransactions;
}

void MyMoneyFile::removePayee(const QString name)
{
  MyMoneyPayee *payee;
  for ( payee=m_payeeList.first(); payee!=0; payee=m_payeeList.next()) {
    if (payee->name() == name) {
      m_payeeList.remove();
      m_dirty=true;
    }
  }
}

int MyMoneyFile::categoryCount(void)
{
  return m_categoryList.count();
}
